package ch.std.jumpstart.config;

import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.caffeine.CaffeineCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.github.benmanes.caffeine.cache.Caffeine;

import ch.std.jumpstart.service.CityServiceCached;

@Configuration
@ConfigurationProperties(prefix = "jumpstart.cache")
@EnableCaching
public class CacheConfiguration {

	Logger logger = LoggerFactory.getLogger(CityServiceCached.class);

	private int initialCapacity = 100;
	private int maximumSize = 500;
	private int ttlMinutes = 10;

	@Bean
	public CacheManager cacheManager() {
		CaffeineCacheManager cacheManager = new CaffeineCacheManager("cities");
		cacheManager.setCaffeine(caffeineCacheBuilder());
		return cacheManager;
	}

	Caffeine<Object, Object> caffeineCacheBuilder() {
		logger.info("caffeine cache initialCapacity = " + this.initialCapacity + ", maximumSize = " + maximumSize + ", ttlMinutes = "  + this.ttlMinutes);
		return Caffeine.newBuilder().initialCapacity(initialCapacity).maximumSize(maximumSize)
				.expireAfterAccess(ttlMinutes, TimeUnit.MINUTES).recordStats(); // do not use weakKeys
	}

	public int getInitialCapacity() {
		return initialCapacity;
	}

	public void setInitialCapacity(int initialCapacity) {
		this.initialCapacity = initialCapacity;
	}

	public int getMaximumSize() {
		return maximumSize;
	}

	public void setMaximumSize(int maximumSize) {
		this.maximumSize = maximumSize;
	}

	public int getTtlMinutes() {
		return ttlMinutes;
	}

	public void setTtlMinutes(int ttlMinutes) {
		this.ttlMinutes = ttlMinutes;
	}
}
